==> k3d_0.5.0.35.orig.tar.gz: k3d-0.5.0.35/modules/scripting/bitmap_modifier_script.cpp <==
// K-3D
// Copyright (c) 1995-2005, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the RenderManScript K-3D object, which can insert scripted data into RenderMan output
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dsdk/create_plugins.h>
#include <k3dsdk/i18n.h>
#include <k3dsdk/iapplication.h>
#include <k3dsdk/iscript_engine.h>
#include <k3dsdk/bitmap_modifier.h>
#include <k3dsdk/module.h>
#include <k3dsdk/node.h>
#include <k3dsdk/persistent.h>
#include <k3dsdk/scripting.h>
#include <k3dsdk/user_property_changed_signal.h>

#define DEFAULT_SCRIPT "#python\n\nimport k3d\n\nk3d.OutputBitmap.assign(k3d.InputBitmap)\n\n"

namespace libk3dscripting
{

/////////////////////////////////////////////////////////////////////////////
// bitmap_modifier_script

class bitmap_modifier_script :
	public k3d::bitmap_modifier<k3d::persistent<k3d::node> >
{
	typedef k3d::bitmap_modifier<k3d::persistent<k3d::node> > base;

public:
	bitmap_modifier_script(k3d::idocument& Document) :
		base(Document),
		m_script(init_owner(*this) + init_name("script") + init_label(_("Script")) + init_description(_("Script that modifies input bitmap")) + init_value<std::string>(DEFAULT_SCRIPT)),
		m_script_engine(0),
		m_user_property_changed_signal(*this)
	{
		m_input.changed_signal().connect(sigc::mem_fun(*this, &bitmap_modifier_script::on_reset_bitmap));
		m_script.changed_signal().connect(sigc::mem_fun(*this, &bitmap_modifier_script::on_reset_bitmap));
		m_user_property_changed_signal.connect(sigc::mem_fun(*this, &bitmap_modifier_script::on_reset_bitmap));
		
		m_output.need_data_signal().connect(sigc::mem_fun(*this, &bitmap_modifier_script::on_create_bitmap));
	}

	~bitmap_modifier_script()
	{
		delete dynamic_cast<k3d::ideletable*>(m_script_engine);
	}

	void on_reset_bitmap()
	{
		m_output.reset();
	}

	k3d::bitmap* on_create_bitmap()
	{
		// If we don't have any input bitmap, we're done ...
		k3d::bitmap* const input = m_input.value();
		if(!input)
			return 0;

		// Otherwise, we create a new output bitmap, ready for modification by the script ...
		k3d::bitmap* const output = new k3d::bitmap();

		// Examine the script to determine the correct language ...
		std::istringstream script(m_script.value());
		k3d::iplugin_factory* const language = k3d::recognize_script_language(script);
		return_val_if_fail(language, output);

		// If the current script engine is for the wrong language, lose it ...
		if(m_script_engine && (m_script_engine->factory().class_id() != language->class_id()))
		{
			delete dynamic_cast<k3d::ideletable*>(m_script_engine);
			m_script_engine = 0;
		}

		// Create our script engine as-needed ...
		if(!m_script_engine)
			m_script_engine = k3d::create_plugin<k3d::iscript_engine>(language->class_id());

		// No script engine?  We're outta here ...
		return_val_if_fail(m_script_engine, output);

		k3d::iscript_engine::context_t context;
		context["Document"] = static_cast<k3d::iunknown*>(&document());
		context["Object"] = static_cast<k3d::iunknown*>(this);
		context["InputBitmap"] = input;
		context["OutputBitmap"] = output;
		return_val_if_fail(m_script_engine->execute(name() + "Script", script, context), output);

		return output;
	}

	k3d::iplugin_factory& factory()
	{
		return get_factory();
	}

	static k3d::iplugin_factory& get_factory()
	{
		static k3d::plugin_factory<
			k3d::document_plugin<bitmap_modifier_script>,
			k3d::interface_list<k3d::ibitmap_sink, k3d::interface_list<k3d::ibitmap_source> > > factory(
			k3d::uuid(0xe6ca5ca4, 0xcfa8452f, 0x8318b2aa, 0xe253956e),
			"BitmapModifierScript",
			_("Bitmap modifier that uses a script to modify an input bitmap"),
			"Scripting Bitmap",
			k3d::iplugin_factory::STABLE);

		return factory;
	}

private:
	k3d_data(std::string, immutable_name, change_signal, with_undo, local_storage, no_constraint, script_property, with_serialization) m_script;
	k3d::iscript_engine* m_script_engine;
	k3d::user_property_changed_signal m_user_property_changed_signal;
};

/////////////////////////////////////////////////////////////////////////////
// bitmap_modifier_script_factory

k3d::iplugin_factory& bitmap_modifier_script_factory()
{
	return bitmap_modifier_script::get_factory();
}

} // namespace libk3dscripting

==> k3d_0.5.12.0.orig.tar.gz: k3d-0.5.12.0/modules/scripting/bitmap_modifier_script.cpp <==
// K-3D
// Copyright (c) 1995-2005, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the RenderManScript K-3D object, which can insert scripted data into RenderMan output
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dsdk/i18n.h>
#include <k3dsdk/bitmap_modifier.h>
#include <k3dsdk/module.h>
#include <k3dsdk/node.h>
#include <k3dsdk/persistent.h>
#include <k3dsdk/scripted_node.h>

#define DEFAULT_SCRIPT "#python\n\nimport k3d\n\nOutput.assign(Input)\n\n"

namespace libk3dscripting
{

/////////////////////////////////////////////////////////////////////////////
// bitmap_modifier_script

class bitmap_modifier_script :
	public k3d::scripted_node<k3d::bitmap_modifier<k3d::persistent<k3d::node> > >
{
	typedef k3d::scripted_node<k3d::bitmap_modifier<k3d::persistent<k3d::node> > > base;

public:
	bitmap_modifier_script(k3d::idocument& Document) :
		base(Document)
	{
		set_script(DEFAULT_SCRIPT);

		m_input.changed_signal().connect(m_output.make_reset_slot());
		connect_script_changed_signal(m_output.make_reset_slot());
	}

	void on_create_bitmap(k3d::bitmap& Bitmap)
	{
		k3d::bitmap* const input_bitmap = m_input.value();
		if(!input_bitmap)
			return;

		k3d::iscript_engine::context_t context;
		context["Document"] = static_cast<k3d::iunknown*>(&document());
		context["Node"] = static_cast<k3d::iunknown*>(this);
		context["Input"] = input_bitmap;
		context["Output"] = &Bitmap;

		execute_script(context);
	}

	k3d::iplugin_factory& factory()
	{
		return get_factory();
	}

	static k3d::iplugin_factory& get_factory()
	{
		static k3d::plugin_factory<
			k3d::document_plugin<bitmap_modifier_script>,
			k3d::interface_list<k3d::ibitmap_sink, k3d::interface_list<k3d::ibitmap_source> > > factory(
			k3d::uuid(0xe6ca5ca4, 0xcfa8452f, 0x8318b2aa, 0xe253956e),
			"BitmapModifierScript",
			_("Bitmap modifier that uses a script to modify an input bitmap"),
			"Scripting Bitmap",
			k3d::iplugin_factory::STABLE);

		return factory;
	}
};

/////////////////////////////////////////////////////////////////////////////
// bitmap_modifier_script_factory

k3d::iplugin_factory& bitmap_modifier_script_factory()
{
	return bitmap_modifier_script::get_factory();
}

} // namespace libk3dscripting
